@@ -3,8 +3,8 @@
 ## -----------------------------------------------------------------------------
 
 sensRange <- function( func, parms = NULL, sensvar = NULL, dist = "unif",
-                       parInput = NULL, parRange = NULL, parMean = NULL, parCovar = NULL,
-                       map = 1, num = 100, ...) {
+         parInput = NULL, parRange = NULL, parMean = NULL, parCovar = NULL,
+         map = 1, num = 100, ...) {
 
   vec2mat <- function(vec) { # make a matrix of a vector
     NN <- names(vec)
@@ -165,7 +165,7 @@ summary.sensRange<-function(object, ...) {
   x    <- attr(object, "x")
   names(x) <- NULL
   nx   <- attr(object, "nx")
-  var  <- attr(object, "var")
+  varnames  <- attr(object, "var")
 
   if (ncol(sens)>1)
     SumSens <- data.frame(
@@ -174,7 +174,7 @@ summary.sensRange<-function(object, ...) {
       Sd   = apply(sens, 2, FUN = sd),
       Min  = apply(sens, 2, FUN = min),
       Max  = apply(sens, 2, FUN = max),
-      q05  = apply(sens, 2, FUN=function(x)quantile(x,probs = 0.05)),
+      q05  = apply(sens, 2, FUN=function(x)quantile(x, probs = 0.05)),
       q25  = apply(sens, 2, FUN=function(x)quantile(x, probs = 0.25)),
       q50  = apply(sens, 2, FUN=function(x)quantile(x, probs = 0.5)),
       q75  = apply(sens, 2, FUN=function(x)quantile(x, probs = 0.75)),
@@ -207,7 +207,7 @@ plot.sensRange<-function(x, xyswap = FALSE, which = NULL,
   ask = NULL, ...) {
   npar <- attr(x, "npar")
   nx   <- attr(x, "nx")
-  var  <- attr(x, "var")
+  varnames  <- attr(x, "var")
   X    <- attr(x, "x")
   sens <- x[, -(1:npar)]
 
@@ -218,7 +218,7 @@ plot.sensRange<-function(x, xyswap = FALSE, which = NULL,
   Ylim   <- is.null(dots$ylim)
   Xlim   <- is.null(dots$xlim)
 
-  Select <- selectvar(which, var, Nall = TRUE)
+  Select <- selectvar(which, varnames, Nall = TRUE)
 
   if (nx > 1) {
     ## Set par mfrow and ask.
@@ -245,24 +245,85 @@ plot.sensRange<-function(x, xyswap = FALSE, which = NULL,
 ## -----------------------------------------------------------------------------
 
 plot.summary.sensRange<-function(x, xyswap = FALSE, which = NULL,
-   legpos = "topleft", col = c(grey(0.8),grey(0.7)), quant = FALSE,
-   ask = NULL, ...) {
+   legpos = "topleft", col = c(grey(0.8), grey(0.7)), quant = FALSE,
+   ask = NULL, obs = NULL, obspar = list(), ...) {
 
   nx    <- attr(x, "nx")
-  var   <- attr(x, "var")
+  varnames   <- attr(x, "var")
 
-  dots   <- list(...)
-  nmdots <- names(dots)
-  Select <- selectvar(which, var, Nall = TRUE)
-  Main   <- is.null(dots$main)
-  Ylim   <- is.null(dots$ylim)
-  Xlim   <- is.null(dots$xlim)
+## check observed data - can be a list
+    nobs <- 0
 
-  dots$ylab <- if(is.null(dots$ylab)) "y" else dots$ylab
-  dots$xlab <- if(is.null(dots$xlab)) "x" else dots$xlab
+    if (! is.null(obs)) {
 
-  Dots <- dots
-  Dots$ylab <- Dots$ylim <- Dots$xlab <- Dots$xlim <- NULL
+      if (!is.data.frame(obs) & is.list(obs)) { # a list with different data sets
+       Obs <- obs
+       obs <- Obs[[1]]  
+       obs.pos <- matrix(nrow = 1, c(1, nrow(obs)))
+       if (! class(obs) %in% c("data.frame", "matrix"))
+         stop ("'obs' should be either a 'data.frame' or a 'matrix'")
+       if (length(Obs) > 1)
+         for ( i in 2 : length(Obs)) {
+           obs <- mergeObs(obs, Obs[[i]])
+           obs.pos <- rbind(obs.pos, c(obs.pos[nrow(obs.pos),2] +1, nrow(obs)))
+         }
+       obsname <- colnames(obs) 
+      } else {
+       if (is.character(obs[,1]) | is.factor(obs[,1]))   # long format - convert
+          obs <- convert2wide(obs)
+       obsname <- colnames(obs) 
+       if (! class(obs) %in% c("data.frame", "matrix"))
+         stop ("'obs' should be either a 'data.frame' or a 'matrix'")
+       obs.pos <- matrix(nrow = 1, c(1, nrow(obs)))
+      }                       
+    DD <- duplicated(obsname)
+    if (sum(DD) > 0)  
+      obs <- mergeObs(obs[,!DD], cbind(obs[,1],obs[,DD]))
+    nobs <- nrow(obs.pos)   
+    }
+
+    dots   <- list(...)
+    nmdots <- names(dots)
+    Which <- which
+    if (is.null (Which) & ! is.null(obs))
+      Which <- which (varnames %in% obsname)
+    
+    Select <- selectvar(Which, varnames, Nall = TRUE)
+    if (length(Select) == 0)
+      stop("There is nothing to plot")
+      
+    np <- length (Select)
+  
+## plotting parameters : split in plot parameters and point parameters
+    plotnames <- c("xlab","ylab","xlim","ylim","main","sub","log","asp",
+                   "ann","axes","frame.plot","panel.first","panel.last",
+                   "cex.lab","cex.axis","cex.main")    
+    
+    # plot.default parameters
+    ii <- names(dots) %in% plotnames
+    dotmain <- dots[ii]
+    dotmain <- setdots(dotmain, np)  # expand to np for each plot
+
+    # these are different from the default
+    dotmain$xlab <- expanddots(dots$xlab, "x"      , np)
+    dotmain$ylab <- expanddots(dots$ylab, "y"      , np)
+    dotmain$main <- expanddots(dots$main, varnames[Select], np)
+
+    # ylim and xlim can be lists and are at least two values
+    yylim  <- expanddotslist(dots$ylim, np)
+    xxlim  <- expanddotslist(dots$xlim, np)
+
+    # point/line parameters
+    ip <- !names(dots) %in% plotnames
+    dotpoints <- dots[ip]
+
+## Position of variables in "obs" (NA = not observed)
+    if (nobs > 0) {
+      ObsWhich <- selectvar(varnames[Select], obsname, NAallowed = TRUE)
+      ObsWhich [ ObsWhich > ncol(obs)] <- NA  
+      Obspar <- setdots(obspar, nobs)
+    } else 
+      ObsWhich <- rep(NA, np)
 
   if (nx > 1)  {    # summary of a times series or a profile...
     ## Set par mfrow and ask.
@@ -274,7 +335,11 @@ plot.summary.sensRange<-function(x, xyswap = FALSE, which = NULL,
       on.exit(devAskNewPage(oask))
     }
 
-    for (i in Select){
+
+    for (ij in 1:np){
+      i <- Select[ij]
+      io <- ObsWhich[ij]   # position of variable in 'obs'
+
       ii <- ((i-1)*nx+1):(i*nx)
       X <- x[ii,]
       if (quant) {
@@ -292,24 +357,57 @@ plot.summary.sensRange<-function(x, xyswap = FALSE, which = NULL,
         xup  <- X$Mean+X$Sd
         leg <- c("Min-Max", "Mean+-sd")
       }
-
-      if (Main) dots$main <- var[i]
-
+    
+      # plotting parameters for deSolve output 1 (opens a plot)
+      Dotmain   <- extractdots(dotmain, i)
+      Dotpoints <- extractdots(dotpoints, 1)
+      
+      Xlog <- Ylog <- FALSE
+      if (! is.null(Dotmain$log)) { 
+        Ylog  <- length(grep("y",Dotmain$log))
+        Xlog  <- length(grep("x",Dotmain$log))
+      }       
       if (!xyswap) {
-        if (Ylim) dots$ylim <- range(cbind(xmin, xmax))
-        if (Xlim) dots$xlim <- range(X$x)
-        do.call("plot", c(alist(X$x, xmean, type = "n"), dots))
+        if (is.null(yylim[[ij]]))
+          Dotmain$ylim <- Range(NULL,c(xmin, xmax, obs[,io]), Ylog)
+        else
+          Dotmain$ylim <- yylim[[ij]]
+        
+        if (is.null(xxlim[[ij]]))
+          Dotmain$xlim <- Range(NULL,X$x,Xlog)
+        else
+          Dotmain$xlim <- xxlim[[ij]]
+        do.call("plot", c(alist(X$x, xmean, type = "n"), Dotmain))
         polygon(c(X$x, rev(X$x)), c(xmin, rev(xmax)), col = col[1], border = NA)
         polygon(c(X$x, rev(X$x)), c(xlow, rev(xup)), col = col[2], border = NA)
-        do.call("lines", c(alist(X$x, xmean), Dots) )
+        do.call("lines", c(alist(X$x, xmean), Dotpoints) )
+        if (! is.na(io))   
+           for (j in 1: nobs) 
+              if (length (i.obs <- obs.pos[j, 1]:obs.pos[j, 2]) > 0) 
+                do.call("points", c(alist(obs[i.obs, 1], obs[i.obs, io]), 
+                         extractdots(Obspar, j) ))        
+        
       } else {
-       if (Xlim) dots$xlim <- range(cbind(xmin, xmax))
-       if (Ylim) dots$ylim <- rev(range(X$x))
-        do.call("plot", c(alist(xmean, X$x, type = "n"), dots))
+        if (is.null(xxlim[[ij]]))
+          Dotmain$xlim <- range(c(xmin, xmax, obs[,io]), na.rm = TRUE)
+        else
+          Dotmain$xlim <- xxlim[[ij]]
+  
+        if (is.null(yylim[[ij]]))
+          Dotmain$ylim <- rev(range(X$x))
+        else
+          Dotmain$ylim <- yylim[[ij]]
+  
+        do.call("plot", c(alist(xmean, X$x, type = "n"), Dotmain))
         polygon(c(xmin, rev(xmax)), c(X$x, rev(X$x)), col = col[1], border = NA)
         polygon(c(xlow, rev(xup)), c(X$x, rev(X$x)), col = col[2], border = NA)
-        do.call("lines", c(alist(xmean, X$x), Dots) )
-      }
+        do.call("lines", c(alist(xmean, X$x), Dotpoints) )
+        if (! is.na(io))   
+           for (j in 1: nobs) 
+              if (length (i.obs <- obs.pos[j, 1]:obs.pos[j, 2]) > 0) 
+                do.call("points", c(alist(obs[i.obs, io], obs[i.obs, 1]), 
+                         extractdots(Obspar, j) ))        
+       }
     } # end i loop
     if (! is.null(legpos))
        legend(legpos, fill = c(col[1], col[2]),
